/***************************************************************************
 *   Copyright (C) 2003 by liuspider                                          *
 *   sharecash@163.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "scimlineedit.h"
#include "qpainter.h"
#include "qdrawutil.h"
#include "qfontmetrics.h"

// #include "qapplication.h"

// #define innerMargin 1
// #include <iostream>
class ScimLineEditPrivate
{
  public:
  ScimLineEditPrivate(){};
  ~ScimLineEditPrivate();
  QSize storedSizeHint;
  int valid_widthhint;
};

ScimLineEdit::ScimLineEdit(QWidget *parent, const char *name)
 : ScimDragableFrame(parent, parent, name), cursorPos(0)
{
    setFrameShape( QFrame::Box );
    setFrameShadow( QFrame::Raised );
    setKeepVisible(false);
    d = new ScimLineEditPrivate();
    d->valid_widthhint = -1;
}
void ScimLineEdit::setText(QString s) 
{
  _text = s; 
  d->valid_widthhint = -1;
  updateGeometry();
}
QSize ScimLineEdit::minimumSizeHint () const
{
  if(d->valid_widthhint == frameWidth())
    return d->storedSizeHint;
    
  d->valid_widthhint = frameWidth();
  QRect br;
  int hextra = 2 * frameWidth();
  int vextra = hextra + 5;
  QFontMetrics fm( fontMetrics() );
  int xw = fm.width( 'x' );
  int m = 0;
  if ( hextra ) // no indent, but we do have a frame
      m = xw / 2 - margin();
  if ( m >= 0 ) {
    hextra += m;
  }
  
  br = fm.boundingRect( 0, 0, 2000 ,2000, AlignAuto | AlignVCenter | ExpandTabs, text() );

  d->storedSizeHint.setWidth( br.width() + hextra);
  d->storedSizeHint.setHeight( br.height() + vextra );

  return d->storedSizeHint;
}
#define PREEDIT_AREA_LEFT_MARGIN 1
void ScimLineEdit::drawContents( QPainter *p )
{
    const QColorGroup& cg = colorGroup();
    QRect cr = contentsRect();
    cr.moveBy(PREEDIT_AREA_LEFT_MARGIN, 2);
    QFontMetrics fm = fontMetrics();

    QBrush bg = QBrush( paletteBackgroundColor() );
    if ( paletteBackgroundPixmap() )
      bg = QBrush( cg.background(), *paletteBackgroundPixmap() );
    else if ( !isEnabled() )
      bg = cg.brush( QColorGroup::Background );

    p->setPen( cg.text() );

    p->setPen( cg.text() );
    p->fillRect( cr, bg );

    //draw text
    unsigned attsize = attribList.size();
    if(attsize)
    {
        if(attribList[0].get_start())
        {
        std::cout << text().mid(0,attribList[0].get_start()-1) << "\n";
        p->drawText( cr, Qt::AlignLeft, text().mid(0,attribList[0].get_start()-1));
        cr.moveBy(fm.width(text(), attribList[0].get_start()),0);
        }
    
        for(unsigned curt = 0; curt < attribList.size(); curt++)
        {
            QString curText = text().mid(attribList[curt].get_start(), attribList[curt].get_length());
            int twidth = fm.width(curText);
            if(attribList[curt].get_type() == scim::SCIM_ATTR_HIGHLIGHT)
            {
                p->save();
                p->fillRect( cr.x(), cr.y(), twidth, fm.height(), cg.highlight() );
                p->setPen( cg.highlightedText() );
                p->drawText( cr, Qt::AlignLeft, curText);
                p->restore();
            }
            else
                p->drawText(cr, Qt::AlignLeft, curText);
            cr.moveBy(twidth,0);
        }
        unsigned finaloffset;
        if(( finaloffset = attribList[attsize-1].get_start() + attribList[attsize-1].get_length()) < text().length())
        {
        p->drawText( cr, Qt::AlignLeft, text().mid(finaloffset));
        }
    }
    else
        p->drawText( cr, Qt::AlignLeft, text());

    //draw cursor
    int cix = fm.width(text(), cursorPosition());
    QPoint from( contentsRect().x() + PREEDIT_AREA_LEFT_MARGIN + cix, cr.top() );
    QPoint to = from + QPoint( 0, fm.height() - frameWidth() );
    p->drawLine( from, to );
}
void ScimLineEdit::setCursorPosition ( int p )
{
    cursorPos = p;
    update();
}
int ScimLineEdit::cursorPosition()
{
    return cursorPos;
}
void ScimLineEdit::setTextFormat(scim::AttributeList a)
{
    attribList = a;
}
ScimLineEdit::~ScimLineEdit()
{
}


#include "scimlineedit.moc"
