/***************************************************************************
 *   Copyright (C) 2003 by liuspider                                          *
 *   sharecash@163.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "scimlistbox.h"
#include <qvariant.h>
#include <qlayout.h>
#include <qlabel.h>

#include <qwidget.h>
#include <qsize.h>

ScimListBox::ScimListBox(QWidget *parent, const char *name, bool _isVertical)
 : ScimDragableFrame(parent, parent, name), curSelection(0)
{
    setFrameShape( QFrame::Box );
    setFrameShadow( QFrame::Raised );
    setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Expanding, 0, 0) );

    QBoxLayout::Direction defaultLayoutDirection = QBoxLayout::LeftToRight;
    
    if(_isVertical)
        defaultLayoutDirection = QBoxLayout::TopToBottom;

    layout = new  QBoxLayout( this, defaultLayoutDirection, 3, 2, "defaultLayout");
    for(int i = 0; i < MAX_CANDIDATES; i++)
    {
        candidatesLbl[i] = new ScimStringListItem(i, this);
        connect(candidatesLbl[i], SIGNAL(selected(int)), this, SLOT(updateSelection(int)));
        connect(candidatesLbl[i], SIGNAL(highlighted(int)), this, SLOT(updateHighlight(int)));
        layout->addWidget(candidatesLbl[i]);
    }   
}
void ScimListBox::setEnableHighlight(bool b)
{
    enableHighlight = b;
    for(int i = 0; i < MAX_CANDIDATES; i++)
      candidatesLbl[i]->setMouseTracking(enableHighlight);
    if(!enableHighlight)
      candidatesLbl[curSelection]->setHighlight(false);
}
void ScimListBox::setVertical(bool b)
{
    QBoxLayout::Direction defaultLayoutDirection = QBoxLayout::LeftToRight;
    if(b)
        defaultLayoutDirection = QBoxLayout::TopToBottom;
    layout->setDirection(defaultLayoutDirection);
}
void ScimListBox::updateHighlight(int _newsel)
{
    if(enableHighlight)
    {
      candidatesLbl[curSelection]->setHighlight(false);
      candidatesLbl[_newsel]->setHighlight(true);
      curSelection = _newsel;
    }
}
void ScimListBox::updateSelection(int i)
{
    emit( itemSelected( i ) );
}
void ScimListBox::updateContent( QStringList &lookupstringlist )
{
    unsigned count = MAX_CANDIDATES < lookupstringlist.size() ? MAX_CANDIDATES : lookupstringlist.size();
    unsigned i = 0;
    for(; i < count; i++)
    {
        candidatesLbl[i]->setText(lookupstringlist[i]);
        candidatesLbl[i]->show();
    }
    for(; i < MAX_CANDIDATES; i++)
    {
        candidatesLbl[i]->clear();
        candidatesLbl[i]->hide();
    }
    adjustSize();
}

ScimListBox::~ScimListBox()
{
}


#include "scimlistbox.moc"
