/***************************************************************************
 *   Copyright (C) 2003 by liuspider                                       *
 *   sharecash@163.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#define USE_SCIM_KDE_SETTINGS
#include "mainwindow.h"

#include <qlayout.h>
#include <qevent.h>
#include <qtoolbutton.h> 
#include <qpopupmenu.h>
#include <ktoolbar.h>
#include <kxmlguifactory.h> 

#include "scimtoolbar.h"
#include "scimmovehandle.h"
#include "src/scimglobalactions.h"
#include "utils/scimxmlguibuilder.cpp"
#include "utils/scimactions.h"

#define MODULE_UUID "985b476c-1a56-47a6-a658-8e0db3be0a7c"
extern "C" {
    static MainWindow* _mainwindow_widget = 0;
    QWidget* init( ModulesCollection* is, ModuleDefinition& md, QWidget* /*preferedParent*/)
    {
        if( md.uuid == MODULE_UUID )
        {
            _mainwindow_widget = new MainWindow(is, 0,0,
                Qt::WStyle_Customize|Qt::WStyle_NoBorder|Qt::WStyle_StaysOnTop|Qt::WX11BypassWM|Qt::WStyle_Tool );
            return _mainwindow_widget;
        }
        return 0;
    }
    bool getInfo( ModulesCollection * , ModuleDefinition* md)
    {
        md->uuid = MODULE_UUID;
        md->type = scim_kde::GeneralModule;
        md->load_type = scim_kde::Permanent;
        md->name = i18n("Main Window");
        md->comment = i18n("Main Toolbar containing all indicators of current input method.");

        return true;
    }
    bool isFinished()
    {
        return !_mainwindow_widget;
    }
}

// #include <ostream>
MainWindow::MainWindow( ModulesCollection* is, QWidget* parent, const char* name, WFlags fl )
    : ScimDragableFrame( parent, parent, name, fl ), ScimXMLGUIClient(is->getActionCollection())
    , PanelTurnedOn( false )
{
    allModules = is;
    InputServer = allModules->getInputServer();
    ScimXMLGUIBuilder* guiBuilder = new ScimXMLGUIBuilder(this);
    KXMLGUIFactory* guiFactory = new KXMLGUIFactory(guiBuilder, this);
    
    if ( !name )
        setName( i18n("Main Window") );
    setLineWidth(1);
    setMidLineWidth(1);
    setFrameShape( QFrame::Box );
    setFrameShadow( QFrame::Raised );
    
    MainWindowLayout = new QHBoxLayout( this, 3, 0, "MainWindowLayout");
    
    moveHandle = new ScimMoveHandle( this, this );
    moveHandle->setKeepVisible( false );
    MainWindowLayout->addWidget( moveHandle );
    
    contextMenu = new QPopupMenu(this, "MainToolBar_RightClickMenu");
    
    toolbar = new ScimToolBar( this, "MainToolBar" );
    toolbar->setEnableContextMenu(true);

    defaultActionCollection = allModules->getActionCollection();

    KToggleAction* StickAction = 
      static_cast<KToggleAction*>(allModules->getActionCollection()->action( "stick_inputwindow" ));
    if(StickAction)
      StickAction->plug(contextMenu);

    QIconSet::setIconSize( QIconSet::Small, QSize( fontMetrics().height(), fontMetrics().height() ) );


    KAction* action = defaultActionCollection->action( "configure" );
    if(action)
      action->plug(contextMenu);
    
    KAction* helpAction = defaultActionCollection->action( "help" );
    if(helpAction)
      helpAction->plug(contextMenu);

    languageChange();

    clearWState( WState_Polished );

    connect( InputServer, SIGNAL( turnOnPanelReq() ), this, SLOT( show() ) );

    move( ScimKdeSettings::mainWindow_POS_X(), ScimKdeSettings::mainWindow_POS_Y());
    

    AutoHideTimeout = ScimKdeSettings::hide_Timeout() * 1000;
    AlwaysShow = ScimKdeSettings::always_Show();
    if( AlwaysShow && AutoHideTimeout > 0 )
    {
        AutoHideTimeoutTimer = new QTimer(this);
        connect( AutoHideTimeoutTimer, SIGNAL( timeout() ), this, SLOT( hide() ) );
    }
    else
    {
        AutoHideTimeoutTimer = 0;
        connect( InputServer, SIGNAL( turnOffPanelReq() ), this, SLOT( hide() ) );
    }
    setXMLFile("mainwindowui.rc");
    MainWindowLayout->addWidget( toolbar );
    guiFactory->addClient(this);
    
    autoSnap = ScimKdeSettings::auto_Snap();
    adjustSize();
//     show();
}

MainWindow::~MainWindow()
{
    ScimKdeSettings::setMainWindow_POS_X(x());
    ScimKdeSettings::setMainWindow_POS_Y(y());
    ScimKdeSettings::writeConfig();
    _mainwindow_widget = 0;
    //std::cout << "Successfully exited.\n" << pos().x() << " " <<pos().y()  << "\n";
}
void MainWindow::adjustSize()
{
    ScimDragableFrame::adjustSize();
    if(autoSnap && !mousePressed)
    {
//     std::cout << "Successfully exited.\n" << screen.x() << " " <<screen.height()  << " " << x() << "\n";
      if(screen.width() - x() - width() < x())
        move(screen.width() - width(), y());
      else
        move(0, y());
    }
}
void MainWindow::contextMenuEvent ( QContextMenuEvent * e)
{
  contextMenu->popup(e->globalPos());
}

void MainWindow::show()
{
    if( AutoHideTimeoutTimer )
        AutoHideTimeoutTimer->start( AutoHideTimeout, true );
    QWidget::show();
    PanelTurnedOn = true;
}

void MainWindow::hide()
{
    if( PanelTurnedOn )
    {
//         if( AutoHideTimeoutTimer && !AutoHideTimeoutTimer->isActive())
//             toolbar->hide();
//         else
            QWidget::hide();
    }
//     serverListMenu->hide();
    PanelTurnedOn = false;
}

void MainWindow::languageChange()
{
    setCaption( i18n( "SCIM Main Window" ) );
//     ServerAction->setText( i18n( "NULL" ) );
}

//ui slot

