/***************************************************************************
 *   Copyright (C) 2003 by liuspider                                          *
 *   sharecash@163.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#define USE_SCIM_KDE_SETTINGS
#include "src/modulescollection.h"
#include "scimlauncher.h"
#include <qtextedit.h> 
#include <qlayout.h>
#include <qtimer.h>

#include <kprocess.h>
#include <klocale.h>

#include <iostream>

#define MODULE_UUID "28efe07e-e62e-4e83-a54b-2417b0fc8f2a"
extern "C" {
    static ScimLauncher* _scimlauncher_widget = 0;
    QWidget* init( ModulesCollection* /*is*/, ModuleDefinition& md, QWidget* /*preferedParent*/)
    {
        if( md.uuid == MODULE_UUID )
        {
            _scimlauncher_widget = new ScimLauncher(/*is,*/);
            return _scimlauncher_widget;
        }
        return 0;
    }
    bool getInfo( ModulesCollection * , ModuleDefinition* md)
    {
        md->uuid = MODULE_UUID;
        md->type = scim_kde::GeneralModule;
        md->load_type = scim_kde::Permanent;
        md->name = i18n("SCIM Launcher");
        md->comment = i18n("Start scim automatically.");

        return true;
    }
    bool isFinished()
    {
        return !_scimlauncher_widget;
    }
}


ScimLauncher::ScimLauncher(QWidget *parent, const char *name)
 : QWidget(parent, name),frontendScim(0)
{
    backendScim = new KProcess(this);
    *backendScim << "scim";
    *backendScim << "-ns" <<  "socket" << "-c" << "socket"/* << "-v" << "-v" << "-v" << "-m" << "frontend"*/;
    //FIXME
    backendScim->setEnvironment("SCIM_CONFIG_SOCKET_ADDRESS","local:/tmp/scim-panel-socket");
    if(ScimKdeSettings::use_Gtk2_Immodule())
    {
      *backendScim << "-f" << "socket";
      frontendScim = new KProcess(this);
      *frontendScim << "scim";
      *frontendScim << "-f" << "x11" << "-s" << "socket" << "-c" << "socket"
       /* << "-v" << "-v" << "-v" << "-m" << "server"*/;
      frontendScim->setEnvironment("SCIM_CONFIG_SOCKET_ADDRESS","local:/tmp/scim-panel-socket");
    } 

    #if ENABLE_DEBUG
    mainLayout = new QVBoxLayout(this);
    mainLayout->setAutoAdd(true);
    textArea = new QTextEdit(this);
    
    connect(backendScim, SIGNAL(receivedStdout(KProcess *, char *, int)), 
      SLOT(receiveStdout(KProcess *, char *, int)));
    connect(backendScim, SIGNAL(receivedStderr(KProcess *, char *, int)), 
      SLOT(receiveStderr(KProcess *, char *, int)));
    show();

    if(!backendScim->start( KProcess::NotifyOnExit, KProcess::AllOutput ))
    #else
    if(!backendScim->start())
    #endif
      std::cerr << "can not start Backend Scim!\n";
    else
      if(frontendScim)
        QTimer::singleShot(500, this, SLOT(startFrontEndSCIM()));
}
void ScimLauncher::startFrontEndSCIM()
{
    if(frontendScim && !frontendScim->isRunning())
    {
        scim::SocketClient client;
        if (client.connect (scim::SocketAddress (scim::String(ScimKdeSettings::_Server_Socket_Address().utf8())))) {
            client.close ();
            if(!frontendScim->start())
              std::cerr << "can not start Frontend Scim!\n";
        }
        else
          QTimer::singleShot(500, this, SLOT(startFrontEndSCIM()));
    }
}
#if ENABLE_DEBUG
void ScimLauncher::receiveStdout(KProcess *proc, char *buffer, int /*buflen*/)
{
    if(proc == backendScim)
    {
       stdout += QString(buffer);
    }
    
    updateText();
}
void ScimLauncher::receiveStderr(KProcess *proc, char *buffer, int /*buflen*/)
{
    if(proc == backendScim)
    {
        stderr += QString(buffer);
    }
    updateText();
}
#else
void ScimLauncher::receiveStdout(KProcess *, char *, int){};
void ScimLauncher::receiveStderr(KProcess *, char *, int){};
#endif
void ScimLauncher::updateText()
{
    #if ENABLE_DEBUG
    if(stderr.isEmpty())
        textArea->setText(stdout);
    else
        textArea->setText(stdout + "----------------STD ERROR OUTPUT-----------------\n" + stderr);   
    #endif
}
ScimLauncher::~ScimLauncher()
{
//     backendScim->disconnect();
//     backendScim->kill();
    _scimlauncher_widget = 0;
}


#include "scimlauncher.moc"
