/* This file is part of the KDE project
   Copyright (C) 2001 Christoph Cullmann <cullmann@kde.org>
   Copyright (C) 2002 Joseph Wenninger <jowenn@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
                                      
#include "configmoduleswidget.h"

// #include "scimsetupwindow.h"
#include <qstringlist.h>
#include <qlayout.h>
#include <qlabel.h>
#include <klocale.h>
#include <qpushbutton.h>
#include <qtooltip.h>
#include <kiconloader.h>
#include <qwhatsthis.h>

//mainwindow, inputwindow, kdesystray
#define DEFAULT_MODULES_UUIDS "985b476c-1a56-47a6-a658-8e0db3be0a7c,\
  a1d34730-6d08-4f59-a418-e63334a46538,664bde5f-76a8-4e18-9600-81c8a64fd2ad" 
class ScimKDEPluginListItem : public QCheckListItem
{
  public:
    ScimKDEPluginListItem(bool & checked, ModuleDefinition& info, QListView *parent);
    ModuleDefinition* info() const { return mInfo; }
  
  protected:	
    void stateChange(bool);
    
  private:
    ModuleDefinition* mInfo;
    bool silentStateChange;
};
         
ScimKDEPluginListItem::ScimKDEPluginListItem(bool & checked, ModuleDefinition& info, QListView *parent)
  : QCheckListItem(parent, info.name, CheckBox)
  , mInfo(&info)
  , silentStateChange(false)
{
  silentStateChange = true;
  setOn(checked);
  silentStateChange = false;
}

void ScimKDEPluginListItem::stateChange(bool b)
{
  if(!silentStateChange)
    static_cast<ScimKDEPluginListView *>(listView())->stateChanged(this, b);
}

ScimKDEPluginListView::ScimKDEPluginListView(QWidget *parent, const char *name)
  : KListView(parent, name)
{
}

void ScimKDEPluginListView::stateChanged(ScimKDEPluginListItem *item, bool b)
{
  emit stateChange(item, b);
}

ScimConfigModulesWidget::ScimConfigModulesWidget(ModulesCollection* is, QWidget *parent) 
  :ScimConfigPluginWidget(parent)
{
  mPluginMan=is;
  (new QVBoxLayout(this,0,0))->setAutoAdd(true);
  listView = new ScimKDEPluginListView(this);
  listView->addColumn(i18n("Name"));
  listView->addColumn(i18n("Comment"));
  QWhatsThis::add(listView,i18n("Here you can see all available scim-panel-kde plugins. Those with a check mark are loaded, and will be loaded again the next time scim-panel-kde is started."));

  connect(listView, SIGNAL(stateChange(ScimKDEPluginListItem *, bool)), 
    this, SLOT(stateChange(ScimKDEPluginListItem *, bool)));
}

void ScimConfigModulesWidget::show()
{
  ScimConfigPluginWidget::show();
  if((int)mPluginMan->availableModules()->count() == listView->childCount())
    return;
  listView->clear();

  for (uint i=0; i<mPluginMan->availableModules()->count(); i++)
  {
    if((*mPluginMan->availableModules()->at(i)).type == scim_kde::GeneralModule)
    {
      bool ch = (*mPluginMan->availableModules()->at(i)).default_on;
      ScimKDEPluginListItem *item = new ScimKDEPluginListItem(ch, 
          (*mPluginMan->availableModules()->at(i)), listView);
      item->setText(0, (*mPluginMan->availableModules()->at(i)).name);
      item->setText(1, (*mPluginMan->availableModules()->at(i)).comment);
      if((*mPluginMan->availableModules()->at(i)).load_type == scim_kde::Temporary)
        item->setVisible(false);
    }
  }
}
 void ScimConfigModulesWidget::stateChange(ScimKDEPluginListItem */*item*/, bool /*b*/)
{   
  emit changed();
}
void ScimConfigModulesWidget::updateWidgetsDefault()
{
    if(isDefault())
      return;
    class ScimKDEPluginListItem * curItem;
    QString defaultModsUUIDs(DEFAULT_MODULES_UUIDS);
    for (int i=0; i < listView->childCount(); i++)
    {
      curItem = static_cast<ScimKDEPluginListItem *>(listView->itemAtIndex(i));
      if( defaultModsUUIDs.contains(curItem->info()->uuid) )
        curItem->setOn(true);
      else
        curItem->setOn(false);
    }
}
void ScimConfigModulesWidget::saveSettings()
{
    if(!hasChanged())
      return;
    QStringList eps;
    class ScimKDEPluginListItem * curItem;
    for (int i=0; i < listView->childCount(); i++)
    {
      curItem = static_cast<ScimKDEPluginListItem *>(listView->itemAtIndex(i));
      if(curItem->info()->type == scim_kde::GeneralModule && curItem->isOn()
        && curItem->info()->load_type == scim_kde::Permanent)
      {
        eps << curItem->info()->uuid;
      }
      curItem->info()->default_on = curItem->isOn();
      curItem->info()->info.should_unload = !curItem->isOn();
      curItem->info()->info.should_reload = curItem->isOn();
    }
    ScimKdeSettings::setEnabledPlugins(eps);
    ScimKdeSettings::writeConfig();
}
bool ScimConfigModulesWidget::hasChanged()
{
    bool changed = false;
    class ScimKDEPluginListItem * curItem;
    for (int i=0; i < listView->childCount(); i++)
    {
      curItem = static_cast<ScimKDEPluginListItem *>(listView->itemAtIndex(i));
      if(curItem->isOn() != curItem->info()->default_on)
      {
        changed = true;
        break;
      }
    }
    return changed;
}

bool ScimConfigModulesWidget::isDefault()
{
    bool isdefault = true;
    class ScimKDEPluginListItem * curItem;
    QString defaultModsUUIDs(DEFAULT_MODULES_UUIDS);
    for (int i=0; i < listView->childCount(); i++)
    {
      curItem = static_cast<ScimKDEPluginListItem *>(listView->itemAtIndex(i));
      if( (curItem->isOn() && !defaultModsUUIDs.contains(curItem->info()->uuid))
        || (!curItem->isOn() && defaultModsUUIDs.contains(curItem->info()->uuid)) )
      {
        isdefault = false;
        break;
      }
    }
    return isdefault;
}
// void ScimConfigPluginWidget::loadPlugin (ScimKDEPluginListItem *item)
// {       
//   mPluginMan->loadPlugin (item->info());
//   mPluginMan->enablePluginGUI (item->info());
//   SettingsDialog->addPluginPage (item->info()->plugin);
//    
//   item->setOn(true);
// }
// 
// void ScimConfigPluginWidget::unloadPlugin (ScimKDEPluginListItem *item)
// {                                  
//   SettingsDialog->removePluginPage (item->info()->plugin);
//   mPluginMan->unloadPlugin (item->info());
//     
//   item->setOn(false);
// }
#include "configmoduleswidget.moc"
