/***************************************************************************
 *   Copyright (C) 2003 by liuspider                                       *
 *   sharecash@163.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#define USE_SCIM_KDE_SETTINGS
#include "modulescollection.h"
#include <dlfcn.h>

#include <qwidget.h>
#include <qtimer.h>
#include <qdir.h>

#include <kapplication.h>
#include <kconfigdialog.h>
#include <kaboutdata.h>

#include "scimglobalactions.h"

typedef bool (*ModuleInInfoFuncPtr)( ModulesCollection * , ModuleDefinition* md);

//FIXME: maybe a user setting is better
#define AUTO_CLEANUP_INTERVAL 200 //in msec
using namespace scim;
ScimGlobalActions* ModulesCollection::getActionCollection()
{
  return defaultActionCollection;
}

ModulesCollection::ModulesCollection( const KAboutData & _aboutData, QObject *, const char *name)
 : QWidget(0, name), aboutData(_aboutData)
{
    InputServer = new scim::SocketServerThread(this);
    kAppMainThread = KApplication::kApplication();
    if(!InputServer->initSocketServer())
    {
      deleteLater();
    }
    else
    {
    InputServer->start();

    defaultActionCollection = new ScimGlobalActions(this, "Global Available Actions");
    modDirs[scim_kde::GeneralModule] << SCIM_KDE_MODULE_ROOT_PATH + QString("plugins");
    modDirs[scim_kde::ConfigModule] << SCIM_KDE_MODULE_ROOT_PATH + QString("setup");
    
    connect(kAppMainThread,SIGNAL(shutDown()),this, SLOT(exitMain()));
    loadModules(false);
    }
}

const KAboutData * ModulesCollection::getAboutData()
{
    return &aboutData;
}

ModulesCollection::~ModulesCollection()
{
    //write settings
    QStringList eps/*, epfs*/;
    for( uint it = 0; it < modulesDef.size() ; it++ )
    {
        if(modulesDef[it].default_on && modulesDef[it].type == scim_kde::GeneralModule
            && modulesDef[it].load_type == scim_kde::Permanent)
        {
            eps << modulesDef[it].uuid;
//             epfs << modulesDef[it].path;
        }
    }
    ScimKdeSettings::setEnabledPlugins(eps);
//     ScimKdeSettings::setEnabledPluginFiles(epfs);

//     InputServer->Disconnect();
    if(InputServer->running())
      InputServer->wait();
//     delete InputServer;
    ScimKdeSettings::writeConfig();
    delete ScimKdeSettings::self();
    SCIM_DEBUG_MAIN (0) << "ModulesCollection exited!\n";
}

void ModulesCollection::exitMain()
{
    connect( this, SIGNAL( destroyed() ), kAppMainThread, SLOT( deref() ));
    connect( this, SIGNAL( allModulesUnloaded() ), InputServer, SLOT( Disconnect() ));
    connect( InputServer, SIGNAL( disconnectCompleted() ), this, SLOT( deleteLater() ));
//     connect( this, SIGNAL( allModulesUnloaded() ), this, SLOT( deleteLater() ));
    unloadAllModules( false );
}
void ModulesCollection::configure()
{
    for( uint it = 0; it < modulesDef.size() ; it++ ) //try to load all KConfigDialog
    {
        if(!modulesDef[it].info.handler && modulesDef[it].uuid == "508e2667-f354-414c-9f1b-eb0d56824528" 
          && modulesDef[it].type == scim_kde::GeneralModule)
            loadModule(it);
    }
    for( uint it = 0; it < modulesDef.size() ; it++ )
    {
        if(modulesDef[it].info.widget && dynamic_cast<KConfigDialog*>(modulesDef[it].info.widget))
        {
            if(modulesDef[it].info.widget->isVisible())
              return;
            modulesDef[it].info.widget->show();
            connect(dynamic_cast<KConfigDialog*>(modulesDef[it].info.widget),SIGNAL(settingsChanged()), 
              this, SLOT( reCalPluginModules() ));
            connect(dynamic_cast<KConfigDialog*>(modulesDef[it].info.widget),SIGNAL(settingsChanged()), 
              this, SIGNAL( settingsChanged() ));
            connect(dynamic_cast<KConfigDialog*>(modulesDef[it].info.widget),SIGNAL(finished()), 
              this, SLOT( cleanupPluginsModules() ));
        }
    }
}
void ModulesCollection::reCalPluginModules()
{  
    uint it = 0;
    bool foundSetup = false, newSetting = false;
    for(; it < modulesDef.size() ; it++ )
    {
        if(modulesDef[it].info.widget && dynamic_cast<KConfigDialog*>(modulesDef[it].info.widget))
        {
            foundSetup = true;
            newSetting = modulesDef[it].default_on;
            break;
        }
    }
    //hack: while setup, do not unload setupwindow even it is not default_on
    if(foundSetup)
      modulesDef[it].default_on = true;
    reviewModules(scim_kde::GeneralModule);
    if(foundSetup)
      modulesDef[it].default_on = newSetting;
}
void ModulesCollection::cleanupPluginsModules()
{
    reviewModules(scim_kde::GeneralModule);
}
void ModulesCollection::reviewModules(scim_kde::ModuleType type)
{
    for( uint it = 0; it < modulesDef.size() ; it++ )
    {
      if(type == modulesDef[it].type)
      {
        if( modulesDef[it].info.widget && !modulesDef[it].default_on )
          modulesDef[it].info.widget->deleteLater();
        else if( !modulesDef[it].info.handler && modulesDef[it].default_on )
          loadModule(it);
      }
    }
}
void ModulesCollection::unLoadModule( QObject * closedm )
{
    bool startTimer = false;
    for( uint it = 0; it < modulesDef.size() ; it++ )
    {
        if( modulesDef[it].info.handler && closedm == (QObject *)modulesDef[it].info.widget)
        {
            modulesDef[it].info.should_unload = true;
            startTimer = true;
            break;
        }
    }
    if(startTimer)
      QTimer::singleShot( AUTO_CLEANUP_INTERVAL, this, SLOT( checkUnneededModules() ) );
}

void ModulesCollection::checkUnneededModules()
{
    bool startTimer = false;
    for( uint it = 0; it < modulesDef.size() ; it++ )
    {
        if( modulesDef[it].info.handler && modulesDef[it].info.should_unload )
        {
            if( modulesDef[it].info.is_widget_exited() )
                closeModule( it );
            else
                startTimer = true;
        }
    }
    if(startTimer)
      QTimer::singleShot( AUTO_CLEANUP_INTERVAL, this, SLOT( checkUnneededModules() ) );
}

void ModulesCollection::closeModule( uint it )
{
    dlclose( modulesDef[it].info.handler );
    modulesDef[it].info.handler = 0;
    modulesDef[it].info.is_widget_exited = 0;
    modulesDef[it].info.widget = 0;
    modulesDef[it].info.should_unload = false;
    SCIM_DEBUG_MAIN (1) << "Unload Module " << modulesDef[it].name << " successfully.\n";
    if( modulesDef[it].info.should_reload )
    {
        SCIM_DEBUG_MAIN (1) << "Now reloading Module " << modulesDef[it].name << "...\n";
        loadModule( modulesDef[it].uuid );
    }

    bool noModulesLoaded = true;
    for( uint it = 0; it < modulesDef.size() ; it++ )
    {
        if( modulesDef[it].info.handler )
        {
            noModulesLoaded = false;
            break;
        }
    }
    if( noModulesLoaded )
        emit( allModulesUnloaded() );
}

scim::SocketServerThread* ModulesCollection::getInputServer()
{
    return InputServer;
}

void ModulesCollection::unloadModule( QString uuid, bool reload )
{
    for( uint it = 0; it < modulesDef.size() ; it++ )
    {
        if( modulesDef[it].info.handler && modulesDef[it].uuid == uuid && modulesDef[it].info.widget )
        {
            modulesDef[it].info.should_reload = reload;
            modulesDef[it].info.widget->deleteLater();
        }
    }
}

void ModulesCollection::unloadAllModules( bool reload )
{
    if( !modulesDef.size() )
      emit allModulesUnloaded();
    for( uint it = 0; it < modulesDef.size() ; it++ )
    {
        if(modulesDef[it].type == scim_kde::GeneralModule &&   //FIXME: currently all the other type of modules will be delete 
             modulesDef[it].info.handler && modulesDef[it].info.widget )
        {
            modulesDef[it].info.should_reload = reload;
            modulesDef[it].info.widget->deleteLater();
        }
//         std::cerr << "NOtify to delete module:" << it << std::endl;
    }
}

bool ModulesCollection::loadModules(bool forceall, scim_kde::ModuleType type, QWidget* preferedParent)
{
    QStringList modList;
    for(uint i = 0; i < modDirs[type].size(); i++)
    {
      QDir defaultLibDir(modDirs[type][i],"*.so");
      if(defaultLibDir.exists()){
      const QFileInfoList *curmodList = defaultLibDir.entryInfoList();
      for ( QFileInfoList::Iterator it = curmodList->begin(); it != curmodList->end(); ++it ) 
          modList.push_back((*it)->absFilePath ());
      }
    }
    if(!modList.size())
    {
      std::cerr << "No plugins of type " << type << " can be found under directories:\n";
      for(uint i = 0; i < modDirs[type].size(); i++)
        std::cerr << modDirs[type][i] << std::endl;
      return false;
    }
    bool success = true;
    struct ModuleDefinition cur_reading_def;
    cur_reading_def.type = type;
    cur_reading_def.loadtype = ModuleDefinition::Lazy;
    cur_reading_def.default_on = false;
    cur_reading_def.info.handler = 0;
    cur_reading_def.info.widget = 0;
    cur_reading_def.info.is_widget_exited = 0;
    cur_reading_def.info.should_unload = false;
    cur_reading_def.info.should_reload = false;

    QValueList<QString> mds;
    if(!forceall)
        mds = ScimKdeSettings::enabledPlugins();
    for ( QStringList::Iterator it = modList.begin(); it != modList.end(); ++it ) 
    {
        if(loadModule(cur_reading_def, *it, mds, preferedParent))
        {
            //FIXME: currently only settings for GeneralModule are saved, 
            // all the other types of modules can only be totally loaded
            
            // so only GeneralModule will can be default_on
            if(modulesDef[modulesDef.size() - 1].type == scim_kde::GeneralModule) 
              modulesDef[modulesDef.size() - 1].default_on = true;
        }
    }

    return success;
}
bool ModulesCollection::loadModule(struct ModuleDefinition defaultMD, const QString & file, 
  const QValueList<QString> & uuidsToLoad, QWidget* preferedParent )
{
    bool reReadInfo = true;
    QValueList<ModuleDefinition>::Iterator it;
    for ( it = modulesDef.begin(); it != modulesDef.end(); ++it ) 
    {
        if((*it).path == defaultMD.path)
        {
            if((*it).info.handler)
            {
              std::cerr << "This module has been opened";
              return false;
            }
            else //it seems we read the info of this module before, so we do not reread info
            {
              reReadInfo = false;
              break;
            }
        }
    }
    defaultMD.path = file;
    if( defaultMD.loadtype != ModuleDefinition::Lazy )
        defaultMD.info.handler = dlopen( file.latin1(), RTLD_NOW );
    else
        defaultMD.info.handler = dlopen( file.latin1(), RTLD_LAZY );
    char * error;
    if ( (error = dlerror()) != NULL ) {
        std::cerr << dlerror();
        return false;
    }
    
    if(reReadInfo)
    {
      ModuleInInfoFuncPtr getInfo = (ModuleInInfoFuncPtr)(dlsym( defaultMD.info.handler, "getInfo" ));
      if ((error = dlerror()) != NULL)  {
          std::cerr << error;
          dlclose( defaultMD.info.handler );
          defaultMD.info.handler = 0;
          return false;
      }
      if (!getInfo(this, &defaultMD) )
          std::cerr << "Can not get Module Info from file " << file << "\n";
    }

    if(uuidsToLoad.size() && !uuidsToLoad.contains(defaultMD.uuid))
    {
      SCIM_DEBUG_MAIN (1) << "Module " << defaultMD.name << " should not be loaded.\n";
      dlclose( defaultMD.info.handler );
      defaultMD.info.handler = 0;
      defaultMD.default_on = false;
      modulesDef.push_back(defaultMD);
      return false;
    }

    //init module
    window_init = (ModuleInitFuncPtr)(dlsym( defaultMD.info.handler, "init" ));
    if ((error = dlerror()) != NULL)  {
        std::cerr << "dlsym init error: " << error << "\n";;
        dlclose( defaultMD.info.handler );
        defaultMD.info.handler = 0;
        return false;
    }

    if(!preferedParent)
      preferedParent = this;
    defaultMD.info.widget = window_init( this, defaultMD, preferedParent );
    
    if( !defaultMD.info.widget )
    {
      dlclose( defaultMD.info.handler );
      defaultMD.info.handler = 0;
      return false;
    }

   defaultMD.info.is_widget_exited = 
      (ModuleIsFinishedFuncPtr)(dlsym( defaultMD.info.handler, "isFinished" ));

    if( defaultMD.info.widget->inherits( "QObject" ) )
    {
        SCIM_DEBUG_MAIN (1) << "Module " << defaultMD.name << " loaded successfully.\n";
//         #if ENABLE_DEBUG
//         if( modulesDef[it].info.should_reload && modulesDef[it].info.widget->inherits( "QWidget" ) )
//             modulesDef[it].info.widget->show();
//         #endif
        connect( defaultMD.info.widget, SIGNAL( destroyed ( QObject * ) ),
            this, SLOT( unLoadModule( QObject * ) ) );
    }
    if(!reReadInfo)
      modulesDef.remove(it);
      
    modulesDef.push_back(defaultMD);

    return true;
}
bool ModulesCollection::loadModule( uint it, QWidget* preferedParent )
{
    if( !modulesDef[it].info.handler )
    {
        struct ModuleDefinition m = modulesDef[it];
        return loadModule(m, modulesDef[it].path, QValueList<QString>(), preferedParent);
    }
    else
    {
        std::cerr << "Warning: module " << modulesDef[it].name << " has already been loaded." << std::endl;
        return false;
    }
}
bool ModulesCollection::loadModule( QString uuid, scim_kde::ModuleType type, QWidget* preferedParent)
{
    for( uint it = 0; it < modulesDef.size() ; it++ )
    {
        if( modulesDef[it].uuid == uuid && modulesDef[it].type == type)
        {
//           struct ModuleDefinition m = modulesDef[it];
          return loadModule(modulesDef[it], modulesDef[it].path, QValueList<QString>(), preferedParent);
        }
    }
    std::cerr << "Error: Can not find module which uuid is " << uuid << std::endl;
    return false;
}

