/***************************************************************************
 *   Copyright (C) 2003 by liuspider                                       *
 *   sharecash@163.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef MODULESCOLLECTION_H
#define MODULESCOLLECTION_H

#include "socketserverthread.h"
// #include "panelconfig.h"
#include <kconfig.h> 
#include <vector>

#include <qstring.h>
#include <qwidget.h>

class ModulesCollection;
namespace scim_kde{
  enum ModuleType{
      GeneralModule,  //will be loaded upon startup
      ConfigModule    //plugins for config modules, used in setupwindow module 
  };
  enum LoadType{
      Permanent,  //if enabled, will be loaded upon startup
      Temporary   //if enabled, will be loaded on-demand
  };
};
typedef bool (*ModuleIsFinishedFuncPtr)();
struct ModuleDefinition{
    QString name;
    QString comment;
    scim_kde::ModuleType type;
    scim_kde::LoadType load_type;
    QString path;
    bool default_on;
    bool temp_on;
    QString uuid;
    enum _loadType{
        Lazy,
        Now
    } loadtype;
    struct RunTimeInfo{
        void* handler;
        QWidget* widget;
        ModuleIsFinishedFuncPtr is_widget_exited;
        bool should_unload;
        bool should_reload;
    } info;
};
typedef QWidget* (*ModuleInitFuncPtr)( ModulesCollection * , ModuleDefinition& md, QWidget* preferedParent);

class KIconLoader;
class KAboutData;
class ScimGlobalActions;

class ModulesCollection : public QWidget
{
Q_OBJECT
public:
    ModulesCollection( const KAboutData &, QObject *parent = 0, const char *name = 0);

    ~ModulesCollection();

    virtual bool loadModule( QString uuid, scim_kde::ModuleType type=scim_kde::GeneralModule, QWidget* preferedParent=0 );
//     bool loadDefaultModule();
    virtual bool loadModules(bool forceall=false, 
      scim_kde::ModuleType type=scim_kde::GeneralModule, QWidget* preferedParent=0);
    QValueList<ModuleDefinition>* availableModules(){ return &modulesDef; };
    virtual scim::SocketServerThread* getInputServer();
    virtual void unloadModule( QString uuid, bool reload = false );
    virtual void unloadAllModules( bool reload = false );
    virtual const KAboutData * getAboutData();
    virtual ScimGlobalActions* getActionCollection();
protected:
    bool loadModule(struct ModuleDefinition defaultMD, const QString & file, 
      const QValueList<QString> & uuidsToLoad, QWidget* preferedParent = 0 );
    bool loadModule( uint index, QWidget* preferedParent = 0 );
signals:
    void allModulesUnloaded();
    void settingsChanged();
public slots:
    void configure();

protected slots:
    void unLoadModule( QObject * );
    void reCalPluginModules();
    void reviewModules(scim_kde::ModuleType type);
    void cleanupPluginsModules();
    void closeModule( uint );
    void checkUnneededModules();
    void exitMain();

private:
    scim::SocketServerThread* InputServer;
    typedef QMap<scim_kde::ModuleType, QStringList> ModulesDirs;
    ModulesDirs modDirs;
    void* mainWindowModuleHandler;
    void* inputWindowModuleHandler;
    const KAboutData & aboutData;
    class KApplication * kAppMainThread;
    ScimGlobalActions* defaultActionCollection;

    ModuleInitFuncPtr window_init;

    QValueList<ModuleDefinition> modulesDef;
    std::vector<uint> deleteQueue;
};

#endif
