/***************************************************************************
 *   Copyright (C) 2003 by liuspider                                          *
 *   sharecash@163.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#define USE_SCIM_KDE_SETTINGS
#include "modulescollection.h"
#include "scimglobalactions.h"
#include <kiconloader.h>
#include <khelpmenu.h>
#include "utils/scimactions.h"
ScimGlobalActions::ScimGlobalActions(QWidget *parent, const char *name)
 : KActionCollection(parent, parent, name), /*mAboutKDE(0),*/ mAboutApp(0), helpDialog(0)
{
  kil = KGlobal::iconLoader();
  
  mc = (ModulesCollection *)QObject::parent ();
  InputServer = mc->getInputServer();
  mHelpMenu = new KHelpMenu( parent );
  connect(mHelpMenu,SIGNAL(showAboutApplication()), this, SLOT(aboutApp()));
//   kmenubar->insertItem(i18n("&Help"), mHelpMenu->menu() );
  new KAction( KGuiItem( i18n( "Reload all modules" ), "reload" ),
    "", this, SLOT(requestReloadAllModules()), this, "reload");
  new KAction( KGuiItem( i18n( "SCIM Help" ), "help" ),
    "", InputServer, SLOT(getHelp()), this, "scim_help");
  KStdAction::help(mHelpMenu, SLOT(appHelpActivated()), this);
  KStdAction::aboutKDE(mHelpMenu,SLOT(aboutKDE ()),this,"aboutkde");
  KStdAction::aboutApp(this,SLOT(aboutApp()),this,"aboutapp");
  connect( InputServer, SIGNAL(showHelp(QString ) ) , this, SLOT(showHelp(QString )));
//   stickIcons.setPixmap(kil->loadIcon("pin_up", KIcon::User), QIconSet::Small, QIconSet::Normal, QIconSet::On);
//   stickIcons.setPixmap(kil->loadIcon("pin_down", KIcon::User), QIconSet::Small, QIconSet::Normal, QIconSet::Off);
  StickAction = new KToggleAction(i18n("Stick"), kil->loadIcon("pin_up", KIcon::User),
                            "", this, SLOT(toggleStick()), this, "stick_inputwindow");
  StickAction->setChecked(ScimKdeSettings::is_Sticky());
  toggleStick();
  LetterWidthAction = new KToggleAction(i18n("Letter"), 
      kil->loadIcon("half_width_letter", KIcon::User),"", InputServer, 
      SLOT(toggleLetterWidth()), this,"toggle_letter_width");
  LetterWidthAction->setToolTip(i18n("Toggle full/half Letter Width"));
  connect( InputServer, SIGNAL(updateFullWidthLetterReq(bool ) ) , this, SLOT(UpdateLetterWidth(bool )));
  
  PuctuateAction = new KToggleAction(i18n("Punctuation"), 
      kil->loadIcon("half_width_punct", KIcon::User), "", InputServer, 
        SLOT(togglePuncWidth()), this, "toggle_punct_width");
  PuctuateAction->setToolTip(i18n("Toggle full/half Punctuation"));
  connect( InputServer, SIGNAL(updateFullWidthPunctuationReq(bool ) ), this, SLOT(UpdatePunctuation(bool )));
  
  new KAction( KGuiItem( i18n( "Configure" ), "configure" ),
         "", mc, SLOT(configure()), this, "configure");

  LangAction = new ScimAction(i18n("Language"), kil->loadIcon("locale", KIcon::Small), "",InputServer, 
      SLOT(changeLang()), this,"change_language");
  LangAction->setOption(scim_kde::ToolBarTextOnly);
  connect( InputServer, SIGNAL(updateStatusStringReq(QString, scim::AttributeList ) ) ,
        this, SLOT(UpdateLangAction(QString, scim::AttributeList )));

  ServerAction = new ScimComboAction(i18n("Input Method"), kil->loadIcon("keyboard", KIcon::Small), "", this, SLOT(changeServer()), this,"change_server");
  connect( InputServer, SIGNAL( updateServerInfoReq( QString, QString ) ), this, SLOT( SetServerInfo( QString, QString ) ) );

  connect( InputServer,
      SIGNAL(showServerMenu(std::vector< QString >, std::vector< QString >, std::vector< QString > )) ,
      this,
      SLOT(receiveServerMenu(std::vector< QString >, std::vector< QString >,  std::vector< QString > )));
//   setToolTip()
  connect( InputServer, SIGNAL( x11FrontEndConnected() ), InputServer, SLOT( getServerList() ) );
  
  connect(mc, SIGNAL(settingsChanged()), SLOT(changeSetting()));
}


ScimGlobalActions::~ScimGlobalActions()
{
    ScimKdeSettings::setIs_Sticky(StickAction->isChecked());
    ScimKdeSettings::writeConfig();
}
void ScimGlobalActions::changeSetting()
{
    StickAction->setChecked(ScimKdeSettings::is_Sticky());
    toggleStick();
}
void ScimGlobalActions::requestReloadAllModules()
{
    mc->unloadAllModules( true );
}
void ScimGlobalActions::UpdateLangAction( QString s, scim::AttributeList )
{
    LangAction->setDisplayedText( s );
}
void ScimGlobalActions::receiveServerMenu(std::vector <QString> _uuids,
        std::vector <QString> names, std::vector <QString> icons)
{
    if(_uuids == uuids)
      return;

    ServerAction->clear();
    //clear image caches
    for(uint i =0; i < pixResources.size(); i++)
    {
      delete pixResources[i].second;
      delete pixResources[i].first;
    }
    pixResources.clear();
    for(uint i = 0; i < names.size(); i++)
    {
      QPixmap *newIcon =new QPixmap(icons[i].ascii());
      pixResources.push_back(pixResType(newIcon,new QIconSet(*newIcon)));
      ServerAction->insertItem(*pixResources[i].second, names[i], i);
    }
    uuids = _uuids;
}

void ScimGlobalActions::changeServer()
{
    if(ServerAction->current() >= 0 && static_cast<unsigned>(ServerAction->current()) < uuids.size())
      InputServer->changeServerFactory(uuids[ServerAction->current()]);
    else
      std::cerr<< "Selection id " << ServerAction->current() << " in server list menu is out of range.\n";
}
void ScimGlobalActions::SetServerInfo( QString newservername, QString newiconstr )
{
//     ServerNameBtn->setTextLabel( newservername );
//     for(int i = 0; i < ServerAction->count(); i++)
//     {
      ServerAction->setDisplayedText(newservername);
      QPixmap newIcon(newiconstr.ascii());
      QIconSet newIconSet( newIcon );
      ServerAction->setCurrentIconSet(newIconSet/*.pixmap(QIconSet::Small,QIconSet::Normal, QIconSet::On )*/);
//       break;
//     }
    
//     ServerAction->setText( newservername );
//     QPixmap newIcon(newiconstr.ascii());
//     QIconSet newIconSet( newIcon );
//     ServerAction->setIconSet(newIconSet.pixmap(QIconSet::Small,QIconSet::Normal, QIconSet::On ));
//     adjustSize();
}

void ScimGlobalActions::UpdateLetterWidth(bool full)
{
    LetterWidthAction->setChecked(full);
    LetterWidthAction->setIconSet( kil->loadIcon( full ? "full_width_letter": "half_width_letter", KIcon::User) );
}

void ScimGlobalActions::UpdatePunctuation( bool full )
{
    PuctuateAction->setChecked(full);
    PuctuateAction->setIconSet( kil->loadIcon( full ? "full_width_punct": "half_width_punct", KIcon::User) );
}

void ScimGlobalActions::toggleStick()
{
    StickAction->setIconSet( kil->loadIcon( StickAction->isChecked() ? "pin_down": "pin_up", KIcon::User) );
}

// void ScimGlobalActions::aboutKDE()
// {
//   if( mAboutKDE == 0 )
//   {
//     mAboutKDE = new KAboutKDE( mc, "aboutkde", false );
// //     connect( mAboutKDE, SIGNAL(finished()), this, SLOT( dialogFinished()) );
//   }
//   mAboutKDE->show();
// }
void ScimGlobalActions::aboutApp()
{
  if( mAboutApp == 0 )
  {
    mAboutApp = new KAboutApplication( mc->getAboutData(), 
      mc, "about", false );
//     connect( mAboutApp, SIGNAL(finished()), this, SLOT( dialogFinished()) );
  }
  mAboutApp->show();
}
void ScimGlobalActions::showHelp( QString txt)
{
  if( helpDialog == 0 )
    helpDialog = new ScimHelpDialog(mc);

  if( !helpDialog->isVisible() )
  {
      helpDialog->setGeneralInfo( txt );
      helpDialog->show();
  }
  if( helpDialog->isMinimized())
      helpDialog->showNormal();
  else
      helpDialog->raise();
}
// void ScimGlobalActions::configure()
// {
// }
#include "scimglobalactions.moc"
