/***************************************************************************
 *   Copyright (C) 2003 by liuspider                                       *
 *   sharecash@163.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#define USE_SCIM_KDE_SETTINGS
#include "socketserverthread.h"

#include <qstringlist.h>
#include <kapplication.h>

#define SCIM_DEFAULT_SETTINGS_GROUP "SCIM"
using namespace scim;

static void     socket_accept_callback                 (SocketServer   *server,
                                                        const Socket   &client);
static void     socket_receive_callback                (SocketServer   *server,
                                                        const Socket   &client);
static void     socket_exception_callback              (SocketServer   *server,
                                                        const Socket   &client);
static SocketServerThread* _socket_server_thread = 0;

SocketServerThread::SocketServerThread(QObject *parent )
 : QObject(parent), QThread(),x11FrontEndConnectSignalEmitted(false)
{
    SCIM_DEBUG_MAIN (1) << "Initialize Socket Server...\n";

    // Read configurations
    _socket_address = SCIM_PANEL_DEFAULT_SOCKET_ADDRESS;
    _socket_timeout = 500;
    _current_socket_client = -1;
    _current_client_context = 0;
    _last_socket_client = -1;
    _last_client_context = 0;
    _socket_client_count = 0;
    _should_exit = false;
    _should_resident = true;
    
    kAppMainThread = KApplication::kApplication();
    
    config = ScimKdeSettings::self()->config();
}
bool SocketServerThread::initSocketServer()
{
    if( !_socket_server.create( SocketAddress( _socket_address ) ) )
    {
        std::cerr << "Cannot create SocketServer.\n";
        return false;
    }
    
    _socket_server_thread = this;
    _socket_server.set_max_clients (1024);

    _socket_server.signal_connect_accept (slot (socket_accept_callback));
    _socket_server.signal_connect_receive (slot (socket_receive_callback));
    _socket_server.signal_connect_exception(slot (socket_exception_callback));

    return true;
}
void SocketServerThread::run()
{
    SCIM_DEBUG_MAIN(1) << " In Socket Server Thread...\n";
    _socket_server.run();
}
void SocketServerThread::AcceptConnection( SocketServer *, const Socket & )
{
    SCIM_DEBUG_MAIN (1) << " Accept Client Connection...\n";
    _socket_client_count++;
}
void SocketServerThread::ReceiveData( SocketServer *server, const Socket &client )
{
    int id = client.get_id ();
    int cmd = 0;
    uint32 context = 0;

    /*SCIM_DEBUG_MAIN (2) << " Receive data from client " << id << " ...\n";*/

    if (!check_client_connection (client)) {
        _socket_client_count --;
        server->close_connection (client);
        if(_socket_client_count==0)
        {
          kAppMainThread->lock();
          emit lastConnectoinFinished();
          kAppMainThread->unlock();
        }
        // The focused client is closed.
        if (_current_socket_client == id) {

            _current_socket_client = -1;
            _current_client_context = 0;
            _last_socket_client = -1;
            _last_client_context = 0;
        }

        SCIM_DEBUG_MAIN (1) << "  Client " << id << " closed...\n";

        // Exit panel if there is no connected client remaining.
         if (_socket_client_count <= 0 && !_should_resident) {
             _should_exit = true;
         }

        return;
    }

    if (!_receive_transaction.read_from_socket (client, _socket_timeout))
        return;

    _send_transaction.clear ();
    _send_transaction.put_command (SCIM_TRANS_CMD_REPLY);

    if (
        _receive_transaction.get_data_type () == SCIM_TRANS_DATA_COMMAND &&
        _receive_transaction.get_command (cmd) && cmd == SCIM_TRANS_CMD_REQUEST )
    {
      //FIXME: the context got below is in fact magic key when this is a setting related request
      //             uint32 key;
      // Check the client's magic key.
      // if (_receive_transaction.get_data (key) && validate_client (id, key)){}
      if(_receive_transaction.get_data_type () == SCIM_TRANS_DATA_UINT32 &&
        _receive_transaction.get_data (context)) {
          while (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_COMMAND &&
                _receive_transaction.get_command (cmd)) {
      //moved the line below into if statements where it is required (see below)
      //       _send_transaction.put_data (context);  
          
              if (cmd == SCIM_TRANS_CMD_PANEL_EXIT) {
                _send_transaction.put_data (context);
                  if (_should_exit) {
                      SCIM_DEBUG_MAIN (1) << "  Exit Socket Server Thread.\n";
                      server->shutdown ();
                      SCIM_DEBUG_MAIN (1) << "  Server status: " << server->is_running() << " is still running\n";
                      kAppMainThread->lock();
                      emit(disconnectCompleted());
                      kAppMainThread->unlock();
                  }
                  return;
              } else if (cmd == SCIM_TRANS_CMD_FOCUS_IN) {
                  _send_transaction.put_data (context);
                  _current_socket_client = id;
                  _current_client_context = context;
              } else if ((_current_socket_client == id &&
                          _current_client_context == context) ||
                        (_current_socket_client < 0 &&
                          (_last_socket_client == id &&
                          _last_client_context == context))) {
                  _send_transaction.put_data (context);
                  // Client must focus in before do any other things.
                  if (cmd == SCIM_TRANS_CMD_PANEL_TURN_ON)
                      turn_on_panel();
                  else if (cmd == SCIM_TRANS_CMD_PANEL_TURN_OFF)
                  {
                      turn_off_panel();
                      //FIXME: only after this point, getserverlist can take effort
                      if(!x11FrontEndConnectSignalEmitted) {
                        emit x11FrontEndConnected();  
                        x11FrontEndConnectSignalEmitted = true; 
                      }
                  }
                  else if (cmd == SCIM_TRANS_CMD_PANEL_UPDATE_DISPLAY)
                      update_display ();
                  else if (cmd == SCIM_TRANS_CMD_PANEL_UPDATE_SCREEN)
                      update_screen ();
                  else if (cmd == SCIM_TRANS_CMD_PANEL_UPDATE_SPOT_LOCATION)
                      update_spot_location ();
                  else if (cmd == SCIM_TRANS_CMD_PANEL_UPDATE_SERVER_INFO)
                      update_server_info ();
                  else if (cmd == SCIM_TRANS_CMD_SHOW_PREEDIT_STRING)
                      show_preedit_string ();
                  else if (cmd == SCIM_TRANS_CMD_SHOW_STATUS_STRING)
                      show_status_string ();
                  else if (cmd == SCIM_TRANS_CMD_SHOW_AUX_STRING)
                      show_aux_string ();
                  else if (cmd == SCIM_TRANS_CMD_SHOW_LOOKUP_TABLE)
                      show_lookup_table ();
                  else if (cmd == SCIM_TRANS_CMD_HIDE_PREEDIT_STRING)
                      hide_preedit_string ();
                  else if (cmd == SCIM_TRANS_CMD_HIDE_STATUS_STRING)
                      hide_status_string ();
                  else if (cmd == SCIM_TRANS_CMD_HIDE_AUX_STRING)
                      hide_aux_string ();
                  else if (cmd == SCIM_TRANS_CMD_HIDE_LOOKUP_TABLE)
                      hide_lookup_table ();
                  else if (cmd == SCIM_TRANS_CMD_UPDATE_PREEDIT_STRING)
                      update_preedit_string ();
                  else if (cmd == SCIM_TRANS_CMD_UPDATE_PREEDIT_CARET)
                      update_preedit_caret ();
                  else if (cmd == SCIM_TRANS_CMD_UPDATE_STATUS_STRING)
                      update_status_string ();
                  else if (cmd == SCIM_TRANS_CMD_UPDATE_AUX_STRING)
                      update_aux_string ();
                  else if (cmd == SCIM_TRANS_CMD_UPDATE_LOOKUP_TABLE)
                      update_lookup_table ();
                  else if (cmd == SCIM_TRANS_CMD_UPDATE_FULL_WIDTH_PUNCTUATION)
                      update_full_width_punctuation ();
                  else if (cmd == SCIM_TRANS_CMD_UPDATE_FULL_WIDTH_LETTER)
                      update_full_width_letter ();
                  else if (cmd == SCIM_TRANS_CMD_FOCUS_OUT) {
                      _last_socket_client = _current_socket_client;
                      _last_client_context = _current_client_context;
                      _current_socket_client = -1;
                      _current_client_context = 0;
                      turn_off_panel();
                      break;
                  } else if (cmd == SCIM_TRANS_CMD_PANEL_SHOW_HELP)
                      show_help ();
                  else if (cmd == SCIM_TRANS_CMD_PANEL_SHOW_SERVER_MENU)
                      show_server_menu ();
                  else 
                    SCIM_DEBUG_MAIN (0) << "ERROR: Unknown command: " << cmd << "\n";
              }
              else if (!process_config_request(cmd))
              {
                SCIM_DEBUG_MAIN (1) << " Unknow command: " << cmd << " (in global scope).\n";
              }
          }
      }
      else
      {
        // Check if it's a OPEN_CONNECTION command.
        if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_COMMAND &&
            _receive_transaction.get_command (cmd) &&
            cmd == SCIM_TRANS_CMD_OPEN_CONNECTION) {
                int cmd, key = rand ();
                _send_transaction.clear ();
            
                SCIM_DEBUG_FRONTEND (1) << "socket_open_connection (" << client.get_id () << ").\n";
            
                _send_transaction.put_command (SCIM_TRANS_CMD_REPLY);
                _send_transaction.put_data ((uint32) key);
            
                SCIM_DEBUG_FRONTEND (2) << " Send magic key (" << key << ") to client.\n";
            
                // Send Feedback.
                if (_send_transaction.write_to_socket (client)) {
            
                    SCIM_DEBUG_FRONTEND (2) << " Check client reply.\n";
                    // Check client reply.
                    if (_receive_transaction.read_from_socket (client, _socket_timeout) &&
                        _receive_transaction.get_data_type () == SCIM_TRANS_DATA_COMMAND) {
            
                        if (_receive_transaction.get_command (cmd) &&
                            cmd == SCIM_TRANS_CMD_REPLY) {
                            if (_receive_transaction.get_command (cmd) &&
                                cmd == SCIM_TRANS_CMD_OK) {
                                                //FIXME
          //                                     add_client (client.get_id (), key);
            
                                SCIM_DEBUG_FRONTEND (2) << " New connection created.\n";
            
                                return;
                            }
                        }
                    }
                }
            
                // Client did not pass the registration process, close it.
                SCIM_DEBUG_FRONTEND (2) << " Failed to create new connection.\n";
                server->close_connection (client);
        //                 socket_open_connection (server, client);
            return;
        }
      }
    }
    else
      SCIM_DEBUG_MAIN (1) << " Something unknow happens.\n";

    // cmd == SCIM_TRANS_CMD_REPLY
    _send_transaction.get_command (cmd);

    // the context id
    _send_transaction.get_data (context);

    // Send the reply only if necessary.
    if (_send_transaction.get_data_type () != SCIM_TRANS_DATA_UNKNOWN)
        _send_transaction.write_to_socket (client);
}
void SocketServerThread::ExceptionHandler( SocketServer *server, const Socket &client )
{
    SCIM_DEBUG_MAIN (2) << "  Exception occurred on client " << client.get_id () << " ...\n";

    // The focused client is closed.
    if (_current_socket_client == client.get_id ()) {
        _current_socket_client = -1;
        _current_client_context = 0;
        _last_socket_client = -1;
        _last_client_context = 0;
    }

    _socket_client_count --;

    SCIM_DEBUG_MAIN (1) << "  Client " << client.get_id () << " closed...\n";

    server->close_connection (client);

    return;
}
void SocketServerThread::reset_config()
{
    config->setGroup(SCIM_DEFAULT_SETTINGS_GROUP);
}
bool SocketServerThread::process_config_request( const int & cmd )
{
    String key;
    if (cmd == SCIM_TRANS_CMD_GET_CONFIG_STRING)
    {
      SCIM_DEBUG_FRONTEND (1) << " socket_get_config_string.\n";
  
      if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (key)) {
          String value;
  
          SCIM_DEBUG_FRONTEND (3) << "  Key (" << key << ").\n";
          reset_config();
          if(config->hasKey(QString::fromUtf8(key.c_str())))
          {
            QString s = config->readEntry(QString::fromUtf8(key.c_str()));
            value = s.utf8();
            _send_transaction.put_data (value);
            _send_transaction.put_command (SCIM_TRANS_CMD_OK);
          }
          else
            _send_transaction.put_command (SCIM_TRANS_CMD_FAIL);
      }
      return true;
    }
    else if (cmd == SCIM_TRANS_CMD_GET_CONFIG_INT)
    {
      SCIM_DEBUG_FRONTEND (1) << " socket_get_config_int.\n";
  
      if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (key)) {
  
          SCIM_DEBUG_FRONTEND (3) << "  Key (" << key << ").\n";
  
          reset_config();
          if(config->hasKey(QString::fromUtf8(key.c_str())))
          {
            int value;
            value = config->readNumEntry(QString::fromUtf8(key.c_str()));
            _send_transaction.put_data ((uint32) value);
            _send_transaction.put_command (SCIM_TRANS_CMD_OK);
              
          }
          else
            _send_transaction.put_command (SCIM_TRANS_CMD_FAIL);
      }
      return true;
    }
    else if (cmd == SCIM_TRANS_CMD_GET_CONFIG_DOUBLE)
    {
      SCIM_DEBUG_FRONTEND (1) << " socket_get_config_double.\n";
  
      if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (key)) {
  
          SCIM_DEBUG_FRONTEND (3) << "  Key (" << key << ").\n";
          reset_config();
          if(config->hasKey(QString::fromUtf8(key.c_str())))
          {
            double value = config->readDoubleNumEntry(QString::fromUtf8(key.c_str()));
            char buf [80];
            snprintf (buf, 79, "%lE", value);
            _send_transaction.put_data (String (buf));
            _send_transaction.put_command (SCIM_TRANS_CMD_OK);
            
          }
          else
            _send_transaction.put_command (SCIM_TRANS_CMD_FAIL);
      }
      return true;
    }
    else if (cmd == SCIM_TRANS_CMD_GET_CONFIG_BOOL)
    {
      SCIM_DEBUG_FRONTEND (1) << " socket_get_config_bool.\n";
  
      if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (key)) {
  
          SCIM_DEBUG_FRONTEND (3) << "  Key (" << key << ").\n";
          reset_config();
          if(config->hasKey(QString::fromUtf8(key.c_str())))
          {
            bool value = config->readBoolEntry(QString::fromUtf8(key.c_str()));
            _send_transaction.put_data ((uint32) value);
            _send_transaction.put_command (SCIM_TRANS_CMD_OK);
            SCIM_DEBUG_FRONTEND (3) << "  return (" << value << ").\n";
          }
          else
            _send_transaction.put_command (SCIM_TRANS_CMD_FAIL);
      }
      return true;
    }
    else if (cmd == SCIM_TRANS_CMD_GET_CONFIG_VECTOR_STRING)
    {
      SCIM_DEBUG_FRONTEND (1) << " socket_get_config_vector_string.\n";
  
      if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (key)) {
  
          SCIM_DEBUG_FRONTEND (3) << "  Key (" << key << ").\n";
          reset_config();
          if(config->hasKey(QString::fromUtf8(key.c_str())))
          {
            std::vector <String> vec;
            QStringList ls = config->readListEntry(QString::fromUtf8(key.c_str()));
            vec.resize(ls.size());
            for(uint i = 0; i < ls.size(); i++)
              vec[i]=ls[i].utf8();
            _send_transaction.put_data (vec);
            _send_transaction.put_command (SCIM_TRANS_CMD_OK);
          }
          else
            _send_transaction.put_command (SCIM_TRANS_CMD_FAIL);
      }
      return true;
    }
    else if (cmd == SCIM_TRANS_CMD_GET_CONFIG_VECTOR_INT)
    {
      SCIM_DEBUG_FRONTEND (0) << "SCIM_TRANS_CMD_GET_CONFIG_VECTOR_INT has NOT be implemented .\n";
      return true;
    }
    else 
    if (cmd == SCIM_TRANS_CMD_SET_CONFIG_STRING)
    {
      String value;
  
      SCIM_DEBUG_FRONTEND (1) << " socket_set_config_string.\n";
  
      if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (key) &&
          _receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (value)) {
  
          SCIM_DEBUG_FRONTEND (3) << "  Key   (" << key << ").\n";
          SCIM_DEBUG_FRONTEND (3) << "  Value (" << value << ").\n";
          reset_config();
          config->writeEntry(QString::fromUtf8(key.c_str()), QString::fromUtf8(value.c_str()));
//           if (m_config->write (key, value))
              _send_transaction.put_command (SCIM_TRANS_CMD_OK);
              return true;
      }
    }
    else if (cmd == SCIM_TRANS_CMD_SET_CONFIG_INT)
    {
      uint32 value;
  
      SCIM_DEBUG_FRONTEND (1) << " socket_set_config_int.\n";
  
      if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (key) &&
          _receive_transaction.get_data_type () == SCIM_TRANS_DATA_UINT32 &&
          _receive_transaction.get_data (value)) {
  
          SCIM_DEBUG_FRONTEND (3) << "  Key   (" << key << ").\n";
          SCIM_DEBUG_FRONTEND (3) << "  Value (" << value << ").\n";
          reset_config();
          config->writeEntry(QString::fromUtf8(key.c_str()), (int) value);
//           if (m_config->write (key, (int) value))
              _send_transaction.put_command (SCIM_TRANS_CMD_OK);
              return true;
      }
    }
    else if (cmd == SCIM_TRANS_CMD_SET_CONFIG_DOUBLE)
    {
      String str;
  
      SCIM_DEBUG_FRONTEND (1) << " socket_set_config_double.\n";
  
      if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (key) &&
          _receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (str)) {
          double value;
          sscanf (str.c_str (), "%lE", &value);
  
          SCIM_DEBUG_FRONTEND (3) << "  Key   (" << key << ").\n";
          SCIM_DEBUG_FRONTEND (3) << "  Value (" << value << ").\n";
          reset_config();
          config->writeEntry(QString::fromUtf8(key.c_str()), value);
//           if (m_config->write (key, value))
              _send_transaction.put_command (SCIM_TRANS_CMD_OK);
              return true;
      }
    }
    else if (cmd == SCIM_TRANS_CMD_SET_CONFIG_BOOL)
    {
      uint32 value;
  
      SCIM_DEBUG_FRONTEND (1) << " socket_set_config_bool.\n";
  
      if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (key) &&
          _receive_transaction.get_data_type () == SCIM_TRANS_DATA_UINT32 &&
          _receive_transaction.get_data (value)) {
  
          SCIM_DEBUG_FRONTEND (3) << "  Key   (" << key << ").\n";
          SCIM_DEBUG_FRONTEND (3) << "  Value (" << value << ").\n";
          reset_config();
          config->writeEntry(QString::fromUtf8(key.c_str()), (bool) value);
//         if (m_config->write (key, (bool) value))
              _send_transaction.put_command (SCIM_TRANS_CMD_OK);
              return true;
      }
    }
    else if (cmd == SCIM_TRANS_CMD_SET_CONFIG_VECTOR_STRING)
    {
      std::vector<String> vec;
  
      SCIM_DEBUG_FRONTEND (1) << " socket_set_config_vector_string.\n";
  
      if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (key) &&
          _receive_transaction.get_data_type () == SCIM_TRANS_DATA_VECTOR_STRING &&
          _receive_transaction.get_data (vec)) {
  
          SCIM_DEBUG_FRONTEND (3) << "  Key (" << key << ").\n";
          reset_config();
          QStringList ls;
          for(uint i = 0; i < vec.size(); i++)
            ls.push_back(QString::fromUtf8(vec[i].c_str()));
          config->writeEntry(QString::fromUtf8(key.c_str()), ls);
  //         if (m_config->write (key, vec))
              _send_transaction.put_command (SCIM_TRANS_CMD_OK);
              return true;
      }
    }
    else if (cmd == SCIM_TRANS_CMD_SET_CONFIG_VECTOR_INT)
    {
      SCIM_DEBUG_FRONTEND (0) << "SCIM_TRANS_CMD_SET_CONFIG_VECTOR_INT has NOT be implemented .\n";
      return true;
    }
    else if (cmd == SCIM_TRANS_CMD_FLUSH_CONFIG)
    {
      config->sync();
      _send_transaction.put_command (SCIM_TRANS_CMD_OK);
      return true;
    }
    else if (cmd == SCIM_TRANS_CMD_ERASE_CONFIG)
    {
      SCIM_DEBUG_FRONTEND (1) << " socket_erase_config.\n";
  
      if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
          _receive_transaction.get_data (key)) {
  
          SCIM_DEBUG_FRONTEND (3) << "  Key   (" << key << ").\n";
          config->deleteEntry(QString::fromUtf8(key.c_str()));
//           if (m_config->erase (key))
              _send_transaction.put_command (SCIM_TRANS_CMD_OK);
              return true;
      }
    }
    
    return false;
}
void SocketServerThread::update_preedit_string()
{
    SCIM_DEBUG_MAIN (1) << "  socket_update_preedit_string...\n";
    String str;
    AttributeList attrs;

    if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
        _receive_transaction.get_data (str) &&
        _receive_transaction.get_data_type () == SCIM_TRANS_DATA_ATTRLIST &&
        _receive_transaction.get_data (attrs)) {
        kAppMainThread->lock();
        emit( updatePreeditStringReq( QString::fromUtf8( str.c_str() ), attrs ) );
        kAppMainThread->unlock();
    }
}
void SocketServerThread::update_spot_location ()
{
    //SCIM_DEBUG_MAIN (3) << "  Set spot location...\n";

    uint32 x, y;

    if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_UINT32 &&
        _receive_transaction.get_data (x) &&
        _receive_transaction.get_data_type () == SCIM_TRANS_DATA_UINT32 &&
        _receive_transaction.get_data (y)) {
        kAppMainThread->lock();
        emit( updateSpotLocationReq( (int)x, (int)y ) );
        kAppMainThread->unlock();
    }
}
void SocketServerThread::update_status_string ()
{
    SCIM_DEBUG_MAIN (1) << "  Update status string...\n";

    String str;
    AttributeList attrs;

    if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
        _receive_transaction.get_data (str) &&
        _receive_transaction.get_data_type () == SCIM_TRANS_DATA_ATTRLIST &&
        _receive_transaction.get_data (attrs)) {
        kAppMainThread->lock();
        emit( updateStatusStringReq( QString::fromUtf8(str.c_str()), attrs ) );
        kAppMainThread->unlock();
    }
}
void SocketServerThread::update_full_width_letter ()
{
    SCIM_DEBUG_MAIN (1) << "  Update full width letter...\n";

    uint32 full;

    if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_UINT32 &&
        _receive_transaction.get_data (full)) {
        kAppMainThread->lock();
        emit( updateFullWidthLetterReq(full) );
        kAppMainThread->unlock();
    }
}

void SocketServerThread::update_full_width_punctuation ()
{
    SCIM_DEBUG_MAIN (1) << "  Update full width punctuation...\n";

    uint32 full;

    if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_UINT32 &&
        _receive_transaction.get_data (full)) {
        kAppMainThread->lock();
        emit( updateFullWidthPunctuationReq( full ) );
        kAppMainThread->unlock();
    }
}

void SocketServerThread::show_preedit_string ()
{
    SCIM_DEBUG_MAIN (1) << "  Show preedit string...\n";
    kAppMainThread->lock();
    emit( showPreeditStringReq() );
    kAppMainThread->unlock();
}

void SocketServerThread::show_status_string ()
{
    SCIM_DEBUG_MAIN (1) << "  Show status string...\n";
    kAppMainThread->lock();
    emit( showStatusStringReq() );
    kAppMainThread->unlock();
}

void SocketServerThread::show_aux_string ()
{
    SCIM_DEBUG_MAIN (1) << "  Show aux string...\n";
    kAppMainThread->lock();
    emit( showAuxStringReq() );
    kAppMainThread->unlock();
}

void SocketServerThread::show_lookup_table ()
{
    SCIM_DEBUG_MAIN (1) << "  Show lookup table...\n";
    kAppMainThread->lock();
    emit( showLookupTableReq() );
    kAppMainThread->unlock();
}

void SocketServerThread::turn_on_panel ()
{
    SCIM_DEBUG_MAIN (1) << "  Turn on panel...\n";
    kAppMainThread->lock();
    emit( turnOnPanelReq() );
    kAppMainThread->unlock();
}

void SocketServerThread::hide_preedit_string ()
{
    SCIM_DEBUG_MAIN (1) << "  Hide preedit string...\n";
    kAppMainThread->lock();
    emit( hidePreeditStringReq() );
    kAppMainThread->unlock();
}

void SocketServerThread::hide_status_string ()
{
    SCIM_DEBUG_MAIN (1) << "  Hide status string...\n";
    kAppMainThread->lock();
    emit( hideStatusStringReq() );
    kAppMainThread->unlock();
}

void SocketServerThread::hide_aux_string ()
{
    SCIM_DEBUG_MAIN (1) << "  Hide aux string...\n";
    kAppMainThread->lock();
    emit( hideAuxStringReq() );
    kAppMainThread->unlock();
}

void SocketServerThread::hide_lookup_table ()
{
    SCIM_DEBUG_MAIN (1) << "  Hide lookup table...\n";
    kAppMainThread->lock();
    emit( hideLookupTableReq() );
    kAppMainThread->unlock();
}

void SocketServerThread::turn_off_panel()
{
    SCIM_DEBUG_MAIN (1) << "  socket_turn_off...\n";
    kAppMainThread->lock();
    emit( turnOffPanelReq() );
    kAppMainThread->unlock();
}
void SocketServerThread::update_display ()
{
    SCIM_DEBUG_MAIN (1) << "  socket_update_display...\n";

    String name;

    if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
        _receive_transaction.get_data (name)) {
        kAppMainThread->lock();
        emit(updateDisplayReq( QString::fromLatin1( name.c_str() ) ) );
        kAppMainThread->unlock();
    }
}

void SocketServerThread::update_screen ()
{
    SCIM_DEBUG_MAIN (1) << "  socket_update_screen...\n";

    uint32 number;

    if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_UINT32 &&
        _receive_transaction.get_data (number)) {
        kAppMainThread->lock();
        emit( updateScreenReq( number ) );
        kAppMainThread->unlock();
    }
}
void SocketServerThread::update_server_info ()
{
    SCIM_DEBUG_MAIN (1) << "  socket_update_server_info...\n";

    String name;
    String iconfile;
	
    if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
        _receive_transaction.get_data (name) &&
	_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
        _receive_transaction.get_data (iconfile)) {
        kAppMainThread->lock();
        emit( updateServerInfoReq( QString::fromUtf8( name.c_str() ), QString( iconfile.c_str() ) ) );
        kAppMainThread->unlock();
    }
}

void SocketServerThread::show_help ()
{
    SCIM_DEBUG_MAIN (1) << "  socket_show_help...\n";
    
    String help;
    if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
        _receive_transaction.get_data (help)) {
        kAppMainThread->lock();
        emit( showHelp( QString::fromUtf8( help.c_str() ) ) );
        kAppMainThread->unlock();
    }
}

void SocketServerThread::show_server_menu ()
{
    SCIM_DEBUG_MAIN (1) << "  socket_show_server_menu...\n";

    std::vector <QString> names;
    std::vector <QString> uuids;
    std::vector <QString> icons;

    String name;
    String uuid;
    String icon;

    while (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
           _receive_transaction.get_data (uuid) &&
           _receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
           _receive_transaction.get_data (name) &&
           _receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
           _receive_transaction.get_data (icon)) {
        uuids.push_back (QString::fromLatin1(uuid.c_str()));
        names.push_back (QString::fromUtf8(name.c_str()));
        icons.push_back (QString(icon.c_str()));
    }
    if(uuids.size())
    {
        kAppMainThread->lock();
        emit( showServerMenu( uuids, names, icons ) );
        kAppMainThread->unlock();
    }
}

void SocketServerThread::update_preedit_caret ()
{
    SCIM_DEBUG_MAIN (1) << "  Update preedit caret...\n";

    uint32 caret;

    if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_UINT32 &&
        _receive_transaction.get_data (caret)) {
        kAppMainThread->lock();
        emit( updatePreeditCaretReq( (int)caret ) );
        kAppMainThread->unlock();
    }
}

void SocketServerThread::update_aux_string ()
{
    SCIM_DEBUG_MAIN (1) << "  Update aux string...\n";

    String str;
    AttributeList attrs;

    if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_STRING &&
        _receive_transaction.get_data (str) &&
        _receive_transaction.get_data_type () == SCIM_TRANS_DATA_ATTRLIST &&
        _receive_transaction.get_data (attrs)) {
        kAppMainThread->lock();
        emit( updateAuxStringReq( QString::fromUtf8(str.c_str()), attrs ) );
        kAppMainThread->unlock();
    }
}

void SocketServerThread::update_lookup_table ()
{
    SCIM_DEBUG_MAIN (1) << "  Update lookup table...\n";

    CommonLookupTable table;
    if (_receive_transaction.get_data_type () == SCIM_TRANS_DATA_LOOKUP_TABLE &&
        _receive_transaction.get_data (table)) {
        size_t i;
        size_t item_num = table.get_current_page_size ();

        KeyEvent       key;
        String         mbs;
        WideString     wcs;
        QStringList    items;

        for (i = 0; i < SCIM_LOOKUP_TABLE_MAX_PAGESIZE; ++ i) {
            if (i < item_num) {
                mbs = String ();

                wcs = table.get_content_in_page (i);

                key = table.get_page_index_key (i);

                if (key.get_ascii_code ())
                    mbs.push_back (key.get_ascii_code ());
                else
                    mbs.push_back (' ');

                mbs += String (". ");
                mbs += utf8_wcstombs (wcs);
                items << QString::fromUtf8( mbs.c_str() );
            }
        }

        _lookup_table_page_up_key = table.get_page_up_key ();
        _lookup_table_page_down_key = table.get_page_down_key ();
        bool preBtnAvailable = true, nextBtnAvailable = true;
        if (!table.get_current_page_start ())
            preBtnAvailable = false;

        if (table.get_current_page_start () + item_num >= table.number_of_entries ())
            nextBtnAvailable = false;
        kAppMainThread->lock();
        emit( updateLookupTableReq( items, preBtnAvailable, nextBtnAvailable ) );
        kAppMainThread->unlock();
        if ( item_num < (uint)table.get_current_page_size ()) {
            _send_transaction.put_command (SCIM_TRANS_CMD_UPDATE_LOOKUP_TABLE_PAGE_SIZE);
            _send_transaction.put_data ((uint32) item_num);
        }
    }
}

bool SocketServerThread::check_client_connection (const Socket &client)
{
    //SCIM_DEBUG_MAIN (3) << "  Check Client Connection...\n";

    unsigned char buf [sizeof(uint32)];

    int nbytes = client.read_with_timeout (buf, sizeof(uint32), _socket_timeout);

    if (nbytes == sizeof (uint32))
        return true;

    if (nbytes < 0) {
        SCIM_DEBUG_MAIN (4) << "   Error occurred when reading socket (" << client.get_id ()
            << "):" << client.get_error_message () << "\n";
    } else {
        SCIM_DEBUG_MAIN (4) << "   Timeout when reading socket (" << client.get_id ()
            << ").\n";
    }

    return false;
}

SocketServerThread::~SocketServerThread()
{
}

void SocketServerThread::Disconnect()
{
    _should_exit = true;

    SocketTransaction trans;
    SocketClient  client;
    SCIM_DEBUG_MAIN (1) << "  Disconnecting with backend (in socket thread)...\n";

    if (client.connect (SocketAddress (_socket_address))) {
        trans.put_command (SCIM_TRANS_CMD_REQUEST);
        trans.put_data ((uint32) 0);
        trans.put_command (SCIM_TRANS_CMD_PANEL_EXIT);
        trans.write_to_socket (client);
        client.close ();
    }
}

void SocketServerThread::selectLookupTableItem(int item)
{
    int client;
    uint32 context;

    if (_current_socket_client >= 0) {
        client = _current_socket_client;
        context = _current_client_context;
    } else {
        client = _last_socket_client;
        context = _last_client_context;
    }

    if (client >= 0) {
        _send_transaction.clear ();
        _send_transaction.put_command (SCIM_TRANS_CMD_REPLY);
        _send_transaction.put_data ((uint32) context);
        _send_transaction.put_command (SCIM_TRANS_CMD_SELECT_LOOKUP_TABLE);
        _send_transaction.put_data ((uint32)item);
        _send_transaction.write_to_socket (Socket (client));
    }
}

void SocketServerThread::lookupTablePageDown()
{
    int client;
    uint32 context;

    if (_current_socket_client >= 0) {
        client = _current_socket_client;
        context = _current_client_context;
    } else {
        client = _last_socket_client;
        context = _last_client_context;
    }

    if (client >= 0) {
        _send_transaction.clear ();
        _send_transaction.put_command (SCIM_TRANS_CMD_REPLY);
        _send_transaction.put_data ((uint32) context);
        _send_transaction.put_command (SCIM_TRANS_CMD_PROCESS_KEYEVENT);
        _send_transaction.put_data (_lookup_table_page_down_key);
        _send_transaction.write_to_socket (Socket (client));
    }
}
void SocketServerThread::lookupTablePageUp()
{
    int client;
    uint32 context;

    if (_current_socket_client >= 0) {
        client = _current_socket_client;
        context = _current_client_context;
    } else {
        client = _last_socket_client;
        context = _last_client_context;
    }

    if (client >= 0) {
        _send_transaction.clear ();
        _send_transaction.put_command (SCIM_TRANS_CMD_REPLY);
        _send_transaction.put_data ((uint32) context);
        _send_transaction.put_command (SCIM_TRANS_CMD_PROCESS_KEYEVENT);
        _send_transaction.put_data (_lookup_table_page_up_key);
        _send_transaction.write_to_socket (Socket (client));
    }
}
void SocketServerThread::getServerList()
{
    int client;
    uint32 context;

    if (_current_socket_client >= 0) {
        client = _current_socket_client;
        context = _current_client_context;
    } else {
        client = _last_socket_client;
        context = _last_client_context;
    }

    if (client >= 0) {
        _send_transaction.clear ();
        _send_transaction.put_command (SCIM_TRANS_CMD_REPLY);
        _send_transaction.put_data ((uint32) context);
        _send_transaction.put_command (SCIM_TRANS_CMD_PANEL_REQUEST_SERVER_MENU);
        _send_transaction.write_to_socket (Socket (client));
    }
}

void SocketServerThread::changeLang()
{
    int client;
    uint32 context;

    if (_current_socket_client >= 0) {
        client = _current_socket_client;
        context = _current_client_context;
    } else {
        client = _last_socket_client;
        context = _last_client_context;
    }

    if (client >= 0) {
        _send_transaction.clear ();
        _send_transaction.put_command (SCIM_TRANS_CMD_REPLY);
        _send_transaction.put_data ((uint32) context);
        _send_transaction.put_command (SCIM_TRANS_CMD_TOGGLE_INPUT_STATUS);
        _send_transaction.write_to_socket (Socket (client));
    }
}
void SocketServerThread::changeServerFactory(const QString &uuid)
{
    int client;
    uint32 context;

    if (_current_socket_client >= 0) {
        client = _current_socket_client;
        context = _current_client_context;
    } else {
        client = _last_socket_client;
        context = _last_client_context;
    }

    if (client >= 0) {
        _send_transaction.clear ();
        _send_transaction.put_command (SCIM_TRANS_CMD_REPLY);
        _send_transaction.put_data ((uint32) context);
        _send_transaction.put_command (SCIM_TRANS_CMD_PANEL_CHANGE_SERVER_FACTORY);
        _send_transaction.put_data (scim::String(uuid.utf8()));
        _send_transaction.write_to_socket (Socket (client));
    }
}
void SocketServerThread::toggleLetterWidth()
{
    int client;
    uint32 context;

    if (_current_socket_client >= 0) {
        client = _current_socket_client;
        context = _current_client_context;
    } else {
        client = _last_socket_client;
        context = _last_client_context;
    }

    if (client >= 0) {
        _send_transaction.clear ();
        _send_transaction.put_command (SCIM_TRANS_CMD_REPLY);
        _send_transaction.put_data ((uint32) context);
        _send_transaction.put_command (SCIM_TRANS_CMD_TOGGLE_FULL_WIDTH_LETTER);
        _send_transaction.write_to_socket (Socket (client));
    }
}

void SocketServerThread::togglePuncWidth()
{
    int client;
    uint32 context;

    if (_current_socket_client >= 0) {
        client = _current_socket_client;
        context = _current_client_context;
    } else {
        client = _last_socket_client;
        context = _last_client_context;
    }

    if (client >= 0) {
        _send_transaction.clear ();
        _send_transaction.put_command (SCIM_TRANS_CMD_REPLY);
        _send_transaction.put_data ((uint32) context);
        _send_transaction.put_command (SCIM_TRANS_CMD_TOGGLE_FULL_WIDTH_PUNCTUATION);
        _send_transaction.write_to_socket (Socket (client));
    }
}
void SocketServerThread::getHelp()
{
    int client;
    uint32 context;

    if (_current_socket_client >= 0) {
        client = _current_socket_client;
        context = _current_client_context;
    } else {
        client = _last_socket_client;
        context = _last_client_context;
    }

    if (client >= 0) {
        _send_transaction.clear ();
        _send_transaction.put_command (SCIM_TRANS_CMD_REPLY);
        _send_transaction.put_data ((uint32) context);
        _send_transaction.put_command (SCIM_TRANS_CMD_PANEL_REQUEST_HELP);
        _send_transaction.write_to_socket (Socket (client));
    }
}
    //_config.reset ();
static void
socket_accept_callback( SocketServer *server, const Socket &client )
{
    _socket_server_thread->AcceptConnection( server, client);
}

static void
socket_receive_callback (SocketServer *server, const Socket &client)
{
    _socket_server_thread->ReceiveData( server, client);
}

static void
socket_exception_callback (SocketServer *server, const Socket &client)
{
    _socket_server_thread->ExceptionHandler( server, client);
}
