/***************************************************************************
 *   Copyright (C) 2003 by liuspider                                          *
 *   sharecash@163.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef SOCKETSERVERTHREAD_H
#define SOCKETSERVERTHREAD_H

#include <sys/wait.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>

#define Uses_C_STDIO
#define Uses_SCIM_LOOKUP_TABLE
#define Uses_SCIM_SOCKET
#define Uses_SCIM_SOCKET_TRANSACTION
#define Uses_SCIM_CONFIG
#define Uses_SCIM_CONFIG_MODULE
#define Uses_SCIM_DEBUG

#include "config.h"
#include <scim-1.0/scim.h>

#include <scim-1.0/scim_panel.h>


#include <qthread.h>
#include <qobject.h>

#ifdef USE_SCIM_KDE_SETTINGS
#include "utils/scimkdesettings.h"
#endif
/**
@author liuspider
*/
namespace scim{
class SocketServerThread : public QObject, public QThread
{
    Q_OBJECT
private:
    SocketServer       _socket_server;
    SocketTransaction  _send_transaction;
    SocketTransaction  _receive_transaction;
    String             _socket_address;
    int                _socket_timeout;

    int                _socket_client_count;

    int                _current_socket_client;
    uint32             _current_client_context;

    int                _last_socket_client;
    uint32             _last_client_context;

    uint32              _check_exit_timeout;
    bool                _should_exit;
    bool                _should_resident;
    KeyEvent           _lookup_table_page_up_key;
    KeyEvent           _lookup_table_page_down_key;
    
    class KApplication * kAppMainThread;

private:
    bool x11FrontEndConnectSignalEmitted;
    void turn_on_panel ();
    void turn_off_panel();
    void update_display ();
    void update_screen ();
    void update_server_info ();
    void update_spot_location ();
    void show_preedit_string ();
    void show_status_string ();
    void show_aux_string ();
    void show_lookup_table ();
    void hide_preedit_string ();
    void hide_status_string ();
    void hide_aux_string ();
    void hide_lookup_table ();
    void update_preedit_string();
    void update_preedit_caret ();
    void update_status_string ();
    void update_aux_string ();
    void update_lookup_table ();
    void update_full_width_punctuation ();
    void update_full_width_letter ();
    void show_help();
    void show_server_menu();
    bool check_client_connection( const Socket & );
    bool process_config_request( const int & cmd );
    void reset_config();
    class KConfig * config;
signals:
    void x11FrontEndConnected();
    void lastConnectoinFinished();
    void turnOnPanelReq();
    void turnOffPanelReq();
    void updateAuxStringReq( QString, scim::AttributeList );
    void updateDisplayReq( QString );
    void updateScreenReq( uint32 );
    void updateServerInfoReq( QString, QString );
    void updateStatusStringReq( QString, scim::AttributeList );
    void updateFullWidthPunctuationReq( bool );
    void updateFullWidthLetterReq( bool );
    void updateSpotLocationReq( int, int );
    void updatePreeditStringReq( QString, scim::AttributeList );
    void updateLookupTableReq( QStringList, bool, bool );
    void showAuxStringReq();
    void showPreeditStringReq();
    void showStatusStringReq();
    void showServerMenu(std::vector <QString>, std::vector <QString>, std::vector <QString>);
    void showLookupTableReq();
    void showHelp( QString );
    void hidePreeditStringReq();
    void hideStatusStringReq();
    void hideAuxStringReq();
    void hideLookupTableReq();
    void updatePreeditCaretReq( int );
    void disconnectCompleted();
public:
    SocketServerThread( QObject *parent );
    ~SocketServerThread();
    void AcceptConnection( SocketServer *, const Socket & );
    void ReceiveData( SocketServer *, const Socket & );
    void ExceptionHandler( SocketServer *, const Socket & );
    virtual void run();
    bool initSocketServer();
public slots:
    void Disconnect();
    void selectLookupTableItem(int);
    void lookupTablePageUp();
    void lookupTablePageDown();

    virtual void getServerList();
    void changeLang();
    void toggleLetterWidth();
    void togglePuncWidth();
    void getHelp();
    void changeServerFactory(const QString &);
};
};

#endif
