/***************************************************************************
 *   Copyright (C) 2003 by liuspider                                          *
 *   sharecash@163.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "scimactions.h"
#include "scimtoolbar.h"

#include "qtoolbutton.h"

ScimAction::ScimAction(const QString &text, const QIconSet &pix, const KShortcut &cut, 
    const QObject *receiver, const char *slot, KActionCollection *parent, const char *name)
    :KAction (text, pix, cut, receiver, slot, parent, name),option(scim_kde::ToolBarIconUseText){}
ScimAction::ScimAction (const QString &text, const KShortcut &cut, const QObject *receiver, 
    const char *slot, KActionCollection *parent, const char *name)
    :KAction (text, cut, receiver, slot, parent, name){}

ScimAction::~ScimAction()
{
}
// #include <iostream>
void ScimAction::updateDText (int i)
{
    QWidget *w = container( i );
    if ( w->inherits( "ScimToolBar" ) )
    {
        QWidget *button = static_cast<ScimToolBar *>(w)->getWidget( itemId( i ) );
        if ( button->inherits( "QToolButton" ) )
        {
          static_cast<QToolButton *>(button)->setTextLabel( displayText );
          //FIXME: ugly hack
          static_cast<ScimToolBar *>(w)->parentWidget()->adjustSize();
          return;
        }
    }
    KAction::updateText(i);
}
void ScimAction::updateCIconSet( int id )
{
    QWidget *w = container( id );
    int itemId_ = itemId( id );
    if ( w->inherits( "ScimToolBar" ) )
    {
        QWidget *button = static_cast<ScimToolBar *>(w)->getWidget( itemId_ );
        if ( button->inherits( "QToolButton" ) )
        {
          static_cast<QToolButton *>(button)->setIconSet(curIconset);
          //FIXME: ugly hack
          static_cast<ScimToolBar *>(w)->parentWidget()->adjustSize();
          return;
        }
    }
    KAction::updateIconSet(id);
}
void ScimAction::setDisplayedText( QString &t)
{
    displayText = t;
    int len = containerCount();
    for( int id = 0; id < len; ++id )
    {
      updateDText(id);
    }
}
void ScimAction::setCurrentIconSet(QIconSet &q)
{
    curIconset = q;
    int len = containerCount();
    for( int id = 0; id < len; ++id )
    {
      updateCIconSet(id);
    }
}
int ScimAction::plug (QWidget *widget, int index)
{
    if(widget->inherits("ScimToolBar"))
    {
        ScimToolBar *bar = static_cast<ScimToolBar *>( widget );
        int id_ = getToolButtonID();
        KInstance *instance;
        if ( parentCollection() )
          instance = parentCollection()->instance();
        else
          instance = KGlobal::instance();
        QToolButton* newScimTB = 
          new QToolButton(bar,QCString("toolbutton_")+name());
        if((option | scim_kde::ToolBarIconUseText) || (option | scim_kde::ToolBarTextOnly))
        {
          newScimTB->setUsesTextLabel( true );
          newScimTB->setTextPosition( QToolButton::BesideIcon );
        }
        if(!(option | scim_kde::ToolBarTextOnly))
          newScimTB->setIconSet(iconSet());
        bar->insertWidget(id_, 50, newScimTB,index);
        connect(newScimTB, SIGNAL( clicked() ), this, SLOT( slotActivated() ));

        addContainer( bar, id_ );
    
        connect( bar, SIGNAL( destroyed() ), this, SLOT( slotDestroyed() ) );
    
        if ( parentCollection() )
          parentCollection()->connectHighlight( bar, this );
    
        return containerCount() - 1;
    }
    return KAction::plug(widget, index);
}

#include <qpopupmenu.h>

ScimComboAction::ScimComboAction (const QString &text, const QIconSet &pix, const KShortcut &cut,
const QObject *receiver, const char *slot, KActionCollection *parent, const char *name) 
  : ScimAction(text,pix, cut, receiver, slot, parent, name),currentItemIndex(-1){}
ScimComboAction::ScimComboAction (const QString &text, const KShortcut &cut, 
  const QObject *receiver, const char *slot, KActionCollection *parent, const char *name)
  : ScimAction(text,cut, receiver, slot, parent, name),currentItemIndex(-1){}
ScimComboAction::~ScimComboAction(){}
int ScimComboAction::plug (QWidget *widget, int index)
{
    if(widget->inherits("ScimToolBar"))
    {
        ScimToolBar *bar = static_cast<ScimToolBar *>( widget );
        int id_ = getToolButtonID();
        KInstance *instance;
        if ( parentCollection() )
          instance = parentCollection()->instance();
        else
          instance = KGlobal::instance();
        QToolButton* newScimTB = 
          new QToolButton(bar,QCString("toolbutton_")+name());
        QPopupMenu* newPopup = new QPopupMenu(newScimTB);
        for(uint i = 0; i < data.count(); i++)
        {
          newPopup->insertItem(*iconSet(i),text(i), i, i);
        }
        newPopup->setCheckable(true);
        newScimTB->setPopup(newPopup);
        newScimTB->setPopupDelay(1);
        if(option | scim_kde::ToolBarIconUseText)
        {
          newScimTB->setUsesTextLabel( true );
          newScimTB->setTextPosition( QToolButton::BesideIcon );
        }

        bar->insertWidget(id_, 50, newScimTB,index);
        connect(newPopup, SIGNAL( activated(int) ), this, SLOT( setCurrent(int) ));

        addContainer( bar, id_ );
    
        connect( bar, SIGNAL( destroyed() ), this, SLOT( slotDestroyed() ) );
    
        if ( parentCollection() )
          parentCollection()->connectHighlight( bar, this );
    
        return containerCount() - 1;
    }
    return KAction::plug(widget, index);
}
// void ScimComboAction::slotActivated( int index )
// {
//     currentItemIndex = index;
//     
// }
void ScimComboAction::setCurrent( int c) 
{ 
    if( currentItemIndex == c)
      return;
    int oldCurrent = currentItemIndex;
    currentItemIndex = c;

    int len = containerCount();
    for( int id = 0; id < len; ++id )
    {
      QWidget *w = container( id );
      int itemId_ = itemId( id );
      if ( w->inherits( "ScimToolBar" ) )
      {
          QWidget *wdgt = static_cast<ScimToolBar *>(w)->getWidget( itemId_ );
          if ( wdgt->inherits( "QToolButton" ))
          {
            QToolButton* toolb = static_cast<QToolButton *>(wdgt);
//             toolb->setText(text(currentItemIndex));
//             QIconSet newIconSet( *pixmap(currentItemIndex) );
//             toolb->setIconSet(newIconSet.pixmap(QIconSet::Small,QIconSet::Normal, QIconSet::On ));
            QPopupMenu* popup = toolb->popup();
            if(popup)
            {
              if(oldCurrent >= 0)
                popup->setItemChecked(oldCurrent,false);
              popup->setItemChecked(currentItemIndex,true);
            }
          }
      }
    }
    emit activated();
}
void ScimComboAction::clear()
{
    int len = containerCount();
    for( int id = 0; id < len; ++id )
    {
      QWidget *w = container( id );
      int itemId_ = itemId( id );
      if ( w->inherits( "ScimToolBar" ) )
      {
          QWidget *wdgt = static_cast<ScimToolBar *>(w)->getWidget( itemId_ );
          if ( wdgt->inherits( "QToolButton" ) )
          {
            QToolButton *qtb = static_cast<QToolButton *>(wdgt);
            if(qtb->popup())
              qtb->popup()->clear();
          }
      }
    }
    data.clear();
}
// void ScimComboAction::insertItem ( const QString & t, int index)
// {
//     data.insertItem(t,index);
// }
// void ScimComboAction::insertItem ( const QPixmap & pixmap, int index)
// {
//     data.insertItem(pixmap,index);
// }
void ScimComboAction::insertItem ( const QIconSet & icon, const QString & text, int index )
{
    if(index < 0)
      index = count();
    data.insertItem(icon,text,index,index);
    updateData(icon, text, index);
}
// void ScimComboAction::changeItem ( const QString & t, int index )
// {
//     data.changeItem(t,index);
// }
// void ScimComboAction::changeItem ( const QPixmap & im, int index )
// {
//     data.changeItem(im,index);
// }
void ScimComboAction::changeItem ( const QPixmap & im, const QString & t, int index )
{
    data.changeItem(index,im,t);
    updateData(im,t,index);
}
QString ScimComboAction::text ( int index ) const
{
    return data.text(index);
}
const QIconSet * ScimComboAction::iconSet ( int index ) const
{
    return data.iconSet(index);
}
void ScimComboAction::updateData(const QIconSet & im, const QString & t, int index)
{
    int len = containerCount();
    for( int id = 0; id < len; ++id )
    {
      QWidget *w = container( id );
      int itemId_ = itemId( id );
      if ( w->inherits( "ScimToolBar" ) )
      {
          QWidget *wdgt = static_cast<ScimToolBar *>(w)->getWidget( itemId_ );
          if ( wdgt->inherits( "QToolButton" ) )
          {
            QToolButton* qtb = static_cast<QToolButton *>(wdgt);
            if(qtb->popup()->idAt(index) == -1)  //insert a new one
            {
              qtb->popup()->insertItem(im, t, index, index);
            }
            else
            {
              qtb->popup()->changeItem(im, t, index);
            }
          }
      }
    }
}
#include "scimactions.moc"
